@@ -211,6 +211,10 @@ bool Navigation::IsAPIEnabled(JSContext* /* unused */, JSObject* /* unused */) {
 
 void Navigation::Entries(
     nsTArray<RefPtr<NavigationHistoryEntry>>& aResult) const {
+  if (HasEntriesAndEventsDisabled()) {
+    aResult.Clear();
+    return;
+  }
   aResult = mEntries.Clone();
 }
 
@@ -366,9 +370,9 @@ void Navigation::UpdateEntriesForSameDocumentNavigation(
       MOZ_LOG(gNavigationAPILog, LogLevel::Debug, ("Push navigation"));
       mCurrentEntryIndex =
           Some(mCurrentEntryIndex ? *mCurrentEntryIndex + 1 : 0);
-      while (*mCurrentEntryIndex < mEntries.Length()) {
-        disposedEntries.AppendElement(mEntries.PopLastElement());
-      }
+      disposedEntries.AppendElements(Span(mEntries).From(*mCurrentEntryIndex));
+      mEntries.RemoveElementsAt(*mCurrentEntryIndex,
+                                mEntries.Length() - *mCurrentEntryIndex);
       mEntries.AppendElement(MakeRefPtr<NavigationHistoryEntry>(
           GetOwnerGlobal(), aDestinationSHE, *mCurrentEntryIndex));
       break;
@@ -598,7 +602,7 @@ void Navigation::Navigate(JSContext* aCx, const nsAString& aUrl,
 
   RefPtr bc = document->GetBrowsingContext();
   MOZ_DIAGNOSTIC_ASSERT(bc);
-  bc->Navigate(urlRecord, *document->NodePrincipal(),
+  bc->Navigate(urlRecord, document, *document->NodePrincipal(),
                /* per spec, error handling defaults to false */ IgnoreErrors(),
                aOptions.mHistory, /* aNeedsCompletelyLoadedDocument */ false,
                serializedState, apiMethodTracker);
@@ -684,44 +688,55 @@ void Navigation::PerformNavigationTraversal(JSContext* aCx, const nsID& aKey,
 
   // 12. Append the following session history traversal steps to traversable:
   auto* childSHistory = traversable->GetChildSessionHistory();
-  auto performNavigationTraversalSteps =
-      [finished =
-           RefPtr(apiMethodTracker->FinishedPromise())](nsresult aResult) {
-        switch (aResult) {
-          case NS_ERROR_DOM_INVALID_STATE_ERR:
-            // 12.2 Let targetSHE be the session history entry in navigableSHEs
-            //      whose navigation API key is key. If no such entry exists,
-            //      then:
-            finished->MaybeRejectWithInvalidStateError(
-                "No such entry with key found");
-            break;
-          case NS_ERROR_DOM_ABORT_ERR:
-            // 12.5 If result is "canceled-by-beforeunload", then queue a global
-            // task on the navigation and traversal task source given
-            // navigation's relevant global object to reject the finished
-            // promise for apiMethodTracker with a new "AbortError" DOMException
-            // created in navigation's relevant realm.
-            finished->MaybeRejectWithAbortError("Navigation was canceled");
-            break;
-          case NS_ERROR_DOM_SECURITY_ERR:
-            // 12.6 If result is "initiator-disallowed", then queue a global
-            // task on the
-            //      navigation and traversal task source given navigation's
-            //      relevant global object to reject the finished promise for
-            //      apiMethodTracker with a new "SecurityError" DOMException
-            //      created in navigation's relevant realm.
-            finished->MaybeRejectWithSecurityError(
-                "Navigation was not allowed");
-            break;
-          case NS_OK:
-            // 12.3 If targetSHE is navigable's active session history entry,
-            // then abort these steps.
-            break;
-          default:
-            MOZ_DIAGNOSTIC_ASSERT(false, "Unexpected result");
-            break;
-        }
-      };
+  auto performNavigationTraversalSteps = [apiMethodTracker](nsresult aResult) {
+    // 12.3 If targetSHE is navigable's active session history entry,
+    //      then abort these steps.
+    if (NS_SUCCEEDED(aResult)) {
+      return;
+    }
+
+    AutoJSAPI jsapi;
+    if (NS_WARN_IF(!jsapi.Init(
+            apiMethodTracker->mNavigationObject->GetParentObject()))) {
+      return;
+    }
+
+    ErrorResult rv;
+
+    switch (aResult) {
+      case NS_ERROR_DOM_INVALID_STATE_ERR:
+        // 12.2 Let targetSHE be the session history entry in navigableSHEs
+        //      whose navigation API key is key. If no such entry exists,
+        //      then:
+        rv.ThrowInvalidStateError("No such entry with key found");
+        break;
+      case NS_ERROR_DOM_ABORT_ERR:
+        // 12.5 If result is "canceled-by-beforeunload", then queue a global
+        //      task on the navigation and traversal task source given
+        //      navigation's relevant global object to reject the finished
+        //      promise for apiMethodTracker with a new "AbortError"
+        //      DOMException
+        // created in navigation's relevant realm.
+        rv.ThrowAbortError("Navigation was canceled");
+        break;
+      case NS_ERROR_DOM_SECURITY_ERR:
+        // 12.6 If result is "initiator-disallowed", then queue a global task on
+        //      the navigation and traversal task source given navigation's
+        //      relevant global object to reject the finished promise for
+        //      apiMethodTracker with a new "SecurityError" DOMException
+        //      created in navigation's relevant realm.
+        rv.ThrowSecurityError("Navigation was not allowed");
+        break;
+      default:
+        MOZ_DIAGNOSTIC_ASSERT(false, "Unexpected result");
+        rv.ThrowInvalidStateError("Unexpected result");
+        break;
+    }
+    JS::Rooted<JS::Value> rootedExceptionValue(jsapi.cx());
+    MOZ_ALWAYS_TRUE(
+        ToJSValue(jsapi.cx(), std::move(rv), &rootedExceptionValue));
+    apiMethodTracker->RejectFinishedPromise(rootedExceptionValue);
+  };
 
   // 12.4 Let result be the result of applying the traverse history step given
   //      by targetSHE's step to traversable, given sourceSnapshotParams,
@@ -921,6 +936,9 @@ bool Navigation::FireTraverseNavigateEvent(
   // in #fire-a-traverse-navigate-event have been moved to after step 25 in
   // #inner-navigate-event-firing-algorithm in our implementation.
 
+  // Work around for https://github.com/whatwg/html/issues/11802
+  InnerInformAboutAbortingNavigation(aCx);
+
   // Step 5
   RefPtr<NavigationHistoryEntry> destinationNHE =
       FindNavigationHistoryEntry(aDestinationSessionHistoryInfo);
@@ -936,7 +954,7 @@ bool Navigation::FireTraverseNavigateEvent(
           .andThen([](auto& aDocShell) {
             return ToMaybeRef(aDocShell.GetActiveSessionHistoryInfo());
           })
-          .map([aDestinationSessionHistoryInfo](auto& aSessionHistoryInfo) {
+          .map([&aDestinationSessionHistoryInfo](auto& aSessionHistoryInfo) {
             return aDestinationSessionHistoryInfo.SharesDocumentWith(
                 aSessionHistoryInfo);
           })
@@ -1016,6 +1034,7 @@ bool Navigation::FireDownloadRequestNavigateEvent(
   // in #fire-a-download-request-navigate-event have been moved to after step
   // 25 in #inner-navigate-event-firing-algorithm in our implementation.
 
+  // Work around for https://github.com/whatwg/html/issues/11802
   InnerInformAboutAbortingNavigation(aCx);
 
   // Step 3 to step 7
@@ -1139,13 +1158,27 @@ nsresult Navigation::FireEvent(const nsAString& aName) {
 
 static void ExtractErrorInformation(JSContext* aCx,
                                     JS::Handle<JS::Value> aError,
-                                    ErrorEventInit& aErrorEventInitDict) {
+                                    ErrorEventInit& aErrorEventInitDict,
+                                    NavigateEvent* aEvent) {
   nsContentUtils::ExtractErrorValues(
       aCx, aError, aErrorEventInitDict.mFilename, &aErrorEventInitDict.mLineno,
       &aErrorEventInitDict.mColno, aErrorEventInitDict.mMessage);
   aErrorEventInitDict.mError = aError;
   aErrorEventInitDict.mBubbles = false;
   aErrorEventInitDict.mCancelable = false;
+
+  if (!aErrorEventInitDict.mFilename.IsEmpty()) {
+    return;
+  }
+
+  RefPtr document = aEvent->GetAssociatedDocument();
+  if (!document) {
+    return;
+  }
+
+  if (auto* uri = document->GetDocumentURI()) {
+    uri->GetSpec(aErrorEventInitDict.mFilename);
+  }
 }
 
 nsresult Navigation::FireErrorEvent(const nsAString& aName,
@@ -1227,9 +1260,7 @@ struct NavigationWaitForAllScope final : public nsISupports,
     // 5. Abort event given reason.
     if (AutoJSAPI jsapi; !NS_WARN_IF(!jsapi.Init(mEvent->GetParentObject()))) {
       RefPtr navigation = mNavigation;
-      navigation->AbortNavigateEvent(
-          jsapi.cx(), event, aRejectionReason,
-          /*aIsCalledFromNavigateFiringFailureSteps=*/true);
+      navigation->AbortNavigateEvent(jsapi.cx(), event, aRejectionReason);
     }
   }
   // https://html.spec.whatwg.org/#commit-a-navigate-event
@@ -1266,6 +1297,17 @@ struct NavigationWaitForAllScope final : public nsISupports,
 
     // 9. If event's interception state is not "none":
     if (mEvent->InterceptionState() != NavigateEvent::InterceptionState::None) {
+      // The copy of the active session history info might be stale at this
+      // point, so make sure to update that. This is not a spec step, but a side
+      // effect of SHIP owning the session history entries making Navigation API
+      // keep copies for its purposes. Should navigation get aborted at this
+      // point, all we've done is eagerly stored scroll positions.
+      if (RefPtr current = mNavigation->GetCurrentEntry()) {
+        nsPoint scrollPos = docShell->GetCurScrollPos();
+        current->SessionHistoryInfo()->SetScrollPosition(scrollPos.x,
+                                                         scrollPos.y);
+      }
+
       // 5. Set event's interception state to "committed".
       // See https://github.com/whatwg/html/issues/11830 for this change.
       mEvent->SetInterceptionState(NavigateEvent::InterceptionState::Committed);
@@ -1338,6 +1380,11 @@ struct NavigationWaitForAllScope final : public nsISupports,
     if (endResultIsSameDocument) {
       // 10.1 Let promisesList be an empty list.
       AutoTArray<RefPtr<Promise>, 16> promiseList;
+
+      if (StaticPrefs::dom_navigation_api_internal_method_tracker()) {
+        promiseList.AppendElement(mAPIMethodTracker->CommittedPromise());
+      }
+
       // 10.2 For each handler of event's navigation handler list:
       for (auto& handler : mEvent->NavigationHandlerList().Clone()) {
         // 10.2.1 Append the result of invoking handler with an empty
@@ -1383,7 +1430,15 @@ struct NavigationWaitForAllScope final : public nsISupports,
                   scope->CommitNavigateEventSuccessSteps();
                 }
               };
-      if (mAPIMethodTracker) {
+      if (mAPIMethodTracker &&
+          !StaticPrefs::dom_navigation_api_internal_method_tracker()) {
+        // Promise::WaitForAll marks all promises as handled, but since we're
+        // delaying wait for all one microtask, we need to manually mark them
+        // here.
+        for (auto& promise : promiseList) {
+          (void)promise->SetAnyPromiseIsHandled();
+        }
+
         LOG_FMTD("Waiting for committed");
         mAPIMethodTracker->CommittedPromise()
             ->AddCallbacksWithCycleCollectedArgs(
@@ -1416,6 +1471,7 @@ struct NavigationWaitForAllScope final : public nsISupports,
                             mNavigation->mOngoingAPIMethodTracker);
       // Step 11
       mAPIMethodTracker->CleanUp();
+      mNavigation->mOngoingNavigateEvent = nullptr;
     } else {
       // It needs to be ensured that the ongoing navigate event is cleared in
       // every code path (e.g. for download events), so that we don't keep
@@ -1480,6 +1536,19 @@ NS_INTERFACE_MAP_END
 NS_IMPL_CYCLE_COLLECTING_ADDREF(NavigationWaitForAllScope)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(NavigationWaitForAllScope)
 
+already_AddRefed<NavigationAPIMethodTracker> CreateInternalTracker(
+    Navigation* aNavigation) {
+  RefPtr committedPromise =
+      Promise::CreateInfallible(aNavigation->GetOwnerGlobal());
+  (void)committedPromise->SetAnyPromiseIsHandled();
+  RefPtr finishedPromise = Promise::CreateResolvedWithUndefined(
+      aNavigation->GetOwnerGlobal(), IgnoreErrors());
+  return MakeAndAddRef<NavigationAPIMethodTracker>(
+      aNavigation, Nothing(), JS::UndefinedHandleValue,
+      /* aSerializedState */ nullptr,
+      /* aCommittedToEntry */ nullptr, committedPromise, finishedPromise);
+}
+
 // https://html.spec.whatwg.org/#inner-navigate-event-firing-algorithm
 bool Navigation::InnerFireNavigateEvent(
     JSContext* aCx, NavigationType aNavigationType,
@@ -1527,7 +1596,10 @@ bool Navigation::InnerFireNavigateEvent(
   // Step 4
   if (apiMethodTracker) {
     apiMethodTracker->MarkAsNotPending();
+  } else if (StaticPrefs::dom_navigation_api_internal_method_tracker()) {
+    apiMethodTracker = CreateInternalTracker(this);
   }
+
   // This step is currently missing in the spec. See
   // https://github.com/whatwg/html/issues/11816
   mOngoingAPIMethodTracker = apiMethodTracker;
@@ -1759,6 +1831,7 @@ void Navigation::SetCurrentEntryIndex(const SessionHistoryInfo* aTargetInfo) {
   if (auto* entry = FindNavigationHistoryEntry(*aTargetInfo)) {
     MOZ_ASSERT(entry->Index() >= 0);
     mCurrentEntryIndex = Some(entry->Index());
+    return;
   }
 
   LOG_FMTW("Session history entry did not exist");
@@ -1811,28 +1884,21 @@ void Navigation::AbortOngoingNavigation(JSContext* aCx,
   }
 
   // Step 7
-  AbortNavigateEvent(aCx, event, error,
-                     /*aIsCalledFromNavigateFiringFailureSteps=*/false);
+  AbortNavigateEvent(aCx, event, error);
 }
 
 // https://html.spec.whatwg.org/#abort-a-navigateevent
-void Navigation::AbortNavigateEvent(
-    JSContext* aCx, NavigateEvent* aEvent, JS::Handle<JS::Value> aReason,
-    bool aIsCalledFromNavigateFiringFailureSteps) {
+void Navigation::AbortNavigateEvent(JSContext* aCx, NavigateEvent* aEvent,
+                                    JS::Handle<JS::Value> aReason) {
   // 1. Let navigation be event's relevant global object's navigation API.
   // Omitted since this is called from a Navigation object.
 
   // 2. Signal abort on event's abort controller given reason.
-  if (!aIsCalledFromNavigateFiringFailureSteps ||
-      aEvent->InterceptionState() ==
-          NavigateEvent::InterceptionState::Intercepted) {
-    // https://github.com/whatwg/html/issues/11831
-    aEvent->AbortController()->Abort(aCx, aReason);
-  }
+  aEvent->AbortController()->Abort(aCx, aReason);
 
   // 3. Let errorInfo be the result of extracting error information from reason.
   RootedDictionary<ErrorEventInit> init(aCx);
-  ExtractErrorInformation(aCx, aReason, init);
+  ExtractErrorInformation(aCx, aReason, init, aEvent);
 
   // 4. Set navigation's ongoing navigate event to null.
   mOngoingNavigateEvent = nullptr;
