@@ -15,6 +15,7 @@
 #include "mozilla/StaticPrefs_apz.h"
 #include "mozilla/webrender/WebRenderAPI.h"
 #include "nsDisplayList.h"
+#include "nsLayoutUtils.h"
 #include "nsRefreshDriver.h"
 #include "nsStyleStructInlines.h"
 #include "UnitTransforms.h"
@@ -101,7 +102,7 @@ void ClipManager::EndList(const StackingContextHelper& aStackingContext) {
 
 void ClipManager::PushOverrideForASR(const ActiveScrolledRoot* aASR,
                                      const wr::WrSpatialId& aSpatialId) {
-  wr::WrSpatialId space = GetScrollLayer(aASR);
+  wr::WrSpatialId space = GetSpatialId(aASR);
 
   CLIP_LOG("Pushing %p override %zu -> %zu\n", aASR, space.id, aSpatialId.id);
   auto it = mASROverride.insert({space, std::stack<wr::WrSpatialId>()});
@@ -127,7 +128,7 @@ void ClipManager::PopOverrideForASR(const ActiveScrolledRoot* aASR) {
   MOZ_ASSERT(!mCacheStack.empty());
   mCacheStack.pop();
 
-  wr::WrSpatialId space = GetScrollLayer(aASR);
+  wr::WrSpatialId space = GetSpatialId(aASR);
   auto it = mASROverride.find(space);
   if (it == mASROverride.end()) {
     MOZ_ASSERT_UNREACHABLE("Push/PopOverrideForASR should be balanced");
@@ -181,6 +182,7 @@ wr::WrSpaceAndClipChain ClipManager::SwitchItem(nsDisplayListBuilder* aBuilder,
   }
   const ActiveScrolledRoot* asr = aItem->GetActiveScrolledRoot();
   DisplayItemType type = aItem->GetType();
+  const ActiveScrolledRoot* stickyAsr = nullptr;
   if (type == DisplayItemType::TYPE_STICKY_POSITION) {
     // For sticky position items, the ASR is computed differently depending on
     // whether the item has a fixed descendant or not. But for WebRender
@@ -189,13 +191,8 @@ wr::WrSpaceAndClipChain ClipManager::SwitchItem(nsDisplayListBuilder* aBuilder,
     // the sticky item.
     auto* sticky = static_cast<nsDisplayStickyPosition*>(aItem);
     asr = sticky->GetContainerASR();
-
-    // If the leafmost clip for the sticky item is just the displayport clip,
-    // then skip it. This allows sticky items to remain visible even if the
-    // rest of the content in the enclosing scrollframe is checkerboarding.
-    if (sticky->IsClippedToDisplayPort() && clip && clip->mASR == asr) {
-      clip = clip->mParent;
-    }
+    stickyAsr = ActiveScrolledRoot::GetStickyASRFromFrame(sticky->Frame());
+    MOZ_ASSERT(stickyAsr);
   }
 
   CLIP_LOG("processing item %p (%s) asr %p clip %p, inherited = %p\n", aItem,
@@ -247,25 +244,31 @@ wr::WrSpaceAndClipChain ClipManager::SwitchItem(nsDisplayListBuilder* aBuilder,
     clip = clip->mParent;
   }
 
-  // There are two ASR chains here that we need to be fully defined. One is the
-  // ASR chain pointed to by |asr|. The other is the
-  // ASR chain pointed to by clip->mASR. We pick the leafmost
-  // of these two chains because that one will include the other. Calling
-  // DefineScrollLayers with this leafmost ASR will recursively define all the
-  // ASRs that we care about for this item, but will not actually push
-  // anything onto the WR stack.
-  const ActiveScrolledRoot* leafmostASR = asr;
-  if (clip) {
-    leafmostASR = ActiveScrolledRoot::PickDescendant(leafmostASR, clip->mASR);
+  // There are up to three ASR chains here that we need to be fully defined:
+  //  1. The ASR chain pointed to by |asr|
+  //  2. The ASR chain pointed to by clip->mASR
+  //  3. For a sticky item, the ASR chain pointed to by the item's sticky ASR.
+  //     This one is needed so that when we create WebRender commands for the
+  //     sticky item, we can give WebRender accurate information about the
+  //     spatial node we're in.
+  // These chains will often be the same, or one will include the other,
+  // but we can't rely on that always being the case, so we make a
+  // DefineSpatialNodes call on all three. These calls will recursively
+  // define all the ASRs that we care about for this item, but will not
+  // actually push anything onto the WR stack.
+  (void)DefineSpatialNodes(aBuilder, asr, aItem);
+  if (clip && clip->mASR != asr) {
+    (void)DefineSpatialNodes(aBuilder, clip->mASR, aItem);
+  }
+  if (stickyAsr && stickyAsr != asr) {
+    (void)DefineSpatialNodes(aBuilder, stickyAsr, aItem);
   }
-  Maybe<wr::WrSpatialId> leafmostId = DefineScrollLayers(leafmostASR, aItem);
-  Unused << leafmostId;
 
   // Define all the clips in the item's clip chain, and obtain a clip chain id
   // for it.
   clips.mClipChainId = DefineClipChain(clip, auPerDevPixel);
 
-  wr::WrSpatialId space = GetScrollLayer(asr);
+  wr::WrSpatialId space = GetSpatialId(asr);
   clips.mScrollId = SpatialIdAfterOverride(space);
   CLIP_LOG("\tassigning %d -> %d\n", (int)space.id, (int)clips.mScrollId.id);
 
@@ -285,10 +288,14 @@ wr::WrSpaceAndClipChain ClipManager::SwitchItem(nsDisplayListBuilder* aBuilder,
   return spaceAndClipChain;
 }
 
-wr::WrSpatialId ClipManager::GetScrollLayer(const ActiveScrolledRoot* aASR) {
+wr::WrSpatialId ClipManager::GetSpatialId(const ActiveScrolledRoot* aASR) {
   for (const ActiveScrolledRoot* asr = aASR; asr; asr = asr->mParent) {
-    Maybe<wr::WrSpatialId> space =
-        mBuilder->GetScrollIdForDefinedScrollLayer(asr->GetViewId());
+    Maybe<wr::WrSpatialId> space = Nothing();
+    if (asr->mKind == ActiveScrolledRoot::ASRKind::Sticky) {
+      space = mBuilder->GetSpatialIdForDefinedStickyLayer(asr);
+    } else {
+      space = mBuilder->GetScrollIdForDefinedScrollLayer(asr->GetViewId());
+    }
     if (space) {
       return *space;
     }
@@ -304,24 +311,294 @@ wr::WrSpatialId ClipManager::GetScrollLayer(const ActiveScrolledRoot* aASR) {
   return *space;
 }
 
-Maybe<wr::WrSpatialId> ClipManager::DefineScrollLayers(
+StickyScrollContainer* ClipManager::GetStickyScrollContainer(
+    const ActiveScrolledRoot* aASR) {
+  MOZ_ASSERT(aASR->mKind == ActiveScrolledRoot::ASRKind::Sticky);
+  StickyScrollContainer* stickyScrollContainer =
+      StickyScrollContainer::GetOrCreateForFrame(aASR->mFrame);
+  if (stickyScrollContainer) {
+    // If there's no ASR for the scrollframe that this sticky item is attached
+    // to, then don't create a WR sticky item for it either. Trying to do so
+    // will end in sadness because WR will interpret some coordinates as
+    // relative to the nearest enclosing scrollframe, which will correspond
+    // to the nearest ancestor ASR on the gecko side. That ASR will not be the
+    // same as the scrollframe this sticky item is actually supposed to be
+    // attached to, thus the sadness.
+    // Not sending WR the sticky item is ok, because the enclosing scrollframe
+    // will never be asynchronously scrolled. Instead we will always position
+    // the sticky items correctly on the gecko side and WR will never need to
+    // adjust their position itself.
+    if (!stickyScrollContainer->ScrollContainer()
+             ->IsMaybeAsynchronouslyScrolled()) {
+      stickyScrollContainer = nullptr;
+    }
+  }
+  return stickyScrollContainer;
+}
+
+// Returns the smallest distance from "0" to the range [min, max] where
+// min <= max. Despite the name, the return value is actually a 1-D vector,
+// and so may be negative if max < 0.
+static nscoord DistanceToRange(nscoord min, nscoord max) {
+  MOZ_ASSERT(min <= max);
+  if (max < 0) {
+    return max;
+  }
+  if (min > 0) {
+    return min;
+  }
+  MOZ_ASSERT(min <= 0 && max >= 0);
+  return 0;
+}
+
+// Returns the magnitude of the part of the range [min, max] that is greater
+// than zero. The return value is always non-negative.
+static nscoord PositivePart(nscoord min, nscoord max) {
+  MOZ_ASSERT(min <= max);
+  if (min >= 0) {
+    return max - min;
+  }
+  if (max > 0) {
+    return max;
+  }
+  return 0;
+}
+
+// Returns the magnitude of the part of the range [min, max] that is less
+// than zero. The return value is always non-negative.
+static nscoord NegativePart(nscoord min, nscoord max) {
+  MOZ_ASSERT(min <= max);
+  if (max <= 0) {
+    return max - min;
+  }
+  if (min < 0) {
+    return 0 - min;
+  }
+  return 0;
+}
+
+Maybe<wr::WrSpatialId> ClipManager::DefineStickyNode(
+    nsDisplayListBuilder* aBuilder, Maybe<wr::WrSpatialId> aParentSpatialId,
     const ActiveScrolledRoot* aASR, nsDisplayItem* aItem) {
+  nsIFrame* stickyFrame = aASR->mFrame;
+
+  if (Maybe<wr::WrSpatialId> space =
+          mBuilder->GetSpatialIdForDefinedStickyLayer(aASR)) {
+    return space;
+  }
+
+  StickyScrollContainer* stickyScrollContainer = GetStickyScrollContainer(aASR);
+  if (!stickyScrollContainer) {
+    // This may indicated a sticky item that does not need a webrender spatial
+    // node. See the comment in GetStickyScrollContainer for details.
+    return Nothing();
+  }
+
+  // Do not create a spatial node for sticky items with mShouldFlatten=true.
+  // These are inside inactive scroll frames and so cannot move asynchronously.
+  if (stickyScrollContainer->ShouldFlattenAway()) {
+    return Nothing();
+  }
+
+  float auPerDevPixel = stickyFrame->PresContext()->AppUnitsPerDevPixel();
+
+  nsRect itemBounds;
+
+  // Make both itemBounds and scrollPort be relative to the reference frame
+  // of the sticky frame's parent.
+  nsRect scrollPort =
+      stickyScrollContainer->ScrollContainer()->GetScrollPortRect();
+  const nsIFrame* referenceFrame =
+      aBuilder->FindReferenceFrameFor(stickyFrame->GetParent());
+  nsRect transformedBounds = stickyFrame->GetRectRelativeToSelf();
+  DebugOnly transformResult = nsLayoutUtils::TransformRect(
+      stickyFrame, referenceFrame, transformedBounds);
+  MOZ_ASSERT(transformResult == nsLayoutUtils::TRANSFORM_SUCCEEDED);
+  itemBounds = transformedBounds;
+  scrollPort = scrollPort +
+               stickyScrollContainer->ScrollContainer()->GetOffsetToCrossDoc(
+                   referenceFrame);
+
+  Maybe<float> topMargin;
+  Maybe<float> rightMargin;
+  Maybe<float> bottomMargin;
+  Maybe<float> leftMargin;
+  wr::StickyOffsetBounds vBounds = {0.0, 0.0};
+  wr::StickyOffsetBounds hBounds = {0.0, 0.0};
+  nsPoint appliedOffset;
+
+  nsRectAbsolute outer;
+  nsRectAbsolute inner;
+  stickyScrollContainer->GetScrollRanges(stickyFrame, &outer, &inner);
+
+  // The following computations make more sense upon understanding the
+  // semantics of "inner" and "outer", which is explained in the comment on
+  // SetStickyPositionData in Layers.h.
+
+  if (outer.YMost() != inner.YMost()) {
+    // Question: How far will itemBounds.y be from the top of the scrollport
+    // when we have scrolled from the current scroll position of "0" to
+    // reach the range [inner.YMost(), outer.YMost()] where the item gets
+    // stuck?
+    // Answer: the current distance is "itemBounds.y - scrollPort.y". That
+    // needs to be adjusted by the distance to the range, less any other
+    // sticky ranges that fall between 0 and the range. If the distance is
+    // negative (i.e. inner.YMost() <= outer.YMost() < 0) then we would be
+    // scrolling upwards (decreasing scroll offset) to reach that range,
+    // which would increase itemBounds.y and make it farther away from the
+    // top of the scrollport. So in that case the adjustment is -distance.
+    // If the distance is positive (0 < inner.YMost() <= outer.YMost()) then
+    // we would be scrolling downwards, itemBounds.y would decrease, and we
+    // again need to adjust by -distance. If we are already in the range
+    // then no adjustment is needed and distance is 0 so again using
+    // -distance works. If the distance is positive, and the item has both
+    // top and bottom sticky ranges, then the bottom sticky range may fall
+    // (entirely[1] or partly[2]) between the current scroll position.
+    // [1]: 0 <= outer.Y() <= inner.Y() < inner.YMost() <= outer.YMost()
+    // [2]: outer.Y() < 0 <= inner.Y() < inner.YMost() <= outer.YMost()
+    // In these cases, the item doesn't actually move for that part of the
+    // distance, so we need to subtract out that bit, which can be computed
+    // as the positive portion of the range [outer.Y(), inner.Y()].
+    nscoord distance = DistanceToRange(inner.YMost(), outer.YMost());
+    if (distance > 0) {
+      distance -= PositivePart(outer.Y(), inner.Y());
+    }
+    topMargin = Some(NSAppUnitsToFloatPixels(
+        itemBounds.y - scrollPort.y - distance, auPerDevPixel));
+    // Question: What is the maximum positive ("downward") offset that WR
+    // will have to apply to this item in order to prevent the item from
+    // visually moving?
+    // Answer: Since the item is "sticky" in the range [inner.YMost(),
+    // outer.YMost()], the maximum offset will be the size of the range, which
+    // is outer.YMost() - inner.YMost().
+    vBounds.max =
+        NSAppUnitsToFloatPixels(outer.YMost() - inner.YMost(), auPerDevPixel);
+    // Question: how much of an offset has layout already applied to the item?
+    // Answer: if we are
+    // (a) inside the sticky range (inner.YMost() < 0 <= outer.YMost()), or
+    // (b) past the sticky range (inner.YMost() < outer.YMost() < 0)
+    // then layout has already applied some offset to the position of the
+    // item. The amount of the adjustment is |0 - inner.YMost()| in case (a)
+    // and |outer.YMost() - inner.YMost()| in case (b).
+    if (inner.YMost() < 0) {
+      appliedOffset.y = std::min(0, outer.YMost()) - inner.YMost();
+      MOZ_ASSERT(appliedOffset.y > 0);
+    }
+  }
+  if (outer.Y() != inner.Y()) {
+    // Similar logic as in the previous section, but this time we care about
+    // the distance from itemBounds.YMost() to scrollPort.YMost().
+    nscoord distance = DistanceToRange(outer.Y(), inner.Y());
+    if (distance < 0) {
+      distance += NegativePart(inner.YMost(), outer.YMost());
+    }
+    bottomMargin = Some(NSAppUnitsToFloatPixels(
+        scrollPort.YMost() - itemBounds.YMost() + distance, auPerDevPixel));
+    // And here WR will be moving the item upwards rather than downwards so
+    // again things are inverted from the previous block.
+    vBounds.min = NSAppUnitsToFloatPixels(outer.Y() - inner.Y(), auPerDevPixel);
+    // We can't have appliedOffset be both positive and negative, and the top
+    // adjustment takes priority. So here we only update appliedOffset.y if
+    // it wasn't set by the top-sticky case above.
+    if (appliedOffset.y == 0 && inner.Y() > 0) {
+      appliedOffset.y = std::max(0, outer.Y()) - inner.Y();
+      MOZ_ASSERT(appliedOffset.y < 0);
+    }
+  }
+  // Same as above, but for the x-axis
+  if (outer.XMost() != inner.XMost()) {
+    nscoord distance = DistanceToRange(inner.XMost(), outer.XMost());
+    if (distance > 0) {
+      distance -= PositivePart(outer.X(), inner.X());
+    }
+    leftMargin = Some(NSAppUnitsToFloatPixels(
+        itemBounds.x - scrollPort.x - distance, auPerDevPixel));
+    hBounds.max =
+        NSAppUnitsToFloatPixels(outer.XMost() - inner.XMost(), auPerDevPixel);
+    if (inner.XMost() < 0) {
+      appliedOffset.x = std::min(0, outer.XMost()) - inner.XMost();
+      MOZ_ASSERT(appliedOffset.x > 0);
+    }
+  }
+  if (outer.X() != inner.X()) {
+    nscoord distance = DistanceToRange(outer.X(), inner.X());
+    if (distance < 0) {
+      distance += NegativePart(inner.XMost(), outer.XMost());
+    }
+    rightMargin = Some(NSAppUnitsToFloatPixels(
+        scrollPort.XMost() - itemBounds.XMost() + distance, auPerDevPixel));
+    hBounds.min = NSAppUnitsToFloatPixels(outer.X() - inner.X(), auPerDevPixel);
+    if (appliedOffset.x == 0 && inner.X() > 0) {
+      appliedOffset.x = std::max(0, outer.X()) - inner.X();
+      MOZ_ASSERT(appliedOffset.x < 0);
+    }
+  }
+
+  LayoutDeviceRect bounds =
+      LayoutDeviceRect::FromAppUnits(itemBounds, auPerDevPixel);
+  wr::LayoutVector2D applied = {
+      NSAppUnitsToFloatPixels(appliedOffset.x, auPerDevPixel),
+      NSAppUnitsToFloatPixels(appliedOffset.y, auPerDevPixel)};
+  bool needsProp =
+      nsDisplayStickyPosition::ShouldGetStickyAnimationId(stickyFrame);
+  Maybe<wr::WrAnimationProperty> prop;
+  auto displayItemKey = nsDisplayItem::GetPerFrameKey(
+      0, 0, DisplayItemType::TYPE_STICKY_POSITION);
+  auto spatialKey = wr::SpatialKey(uint64_t(stickyFrame), displayItemKey,
+                                   wr::SpatialKeyKind::Sticky);
+  if (needsProp) {
+    RefPtr<WebRenderAPZAnimationData> animationData =
+        mManager->CommandBuilder()
+            .CreateOrRecycleWebRenderUserData<WebRenderAPZAnimationData>(
+                displayItemKey, stickyFrame);
+    uint64_t animationId = animationData->GetAnimationId();
+
+    prop.emplace();
+    prop->id = animationId;
+    prop->key = spatialKey;
+    prop->effect_type = wr::WrAnimationType::Transform;
+  }
+  wr::WrSpatialId spatialId = mBuilder->DefineStickyFrame(
+      aASR, aParentSpatialId, wr::ToLayoutRect(bounds),
+      topMargin.ptrOr(nullptr), rightMargin.ptrOr(nullptr),
+      bottomMargin.ptrOr(nullptr), leftMargin.ptrOr(nullptr), vBounds, hBounds,
+      applied, spatialKey, prop.ptrOr(nullptr));
+
+  return Some(spatialId);
+}
+
+Maybe<wr::WrSpatialId> ClipManager::DefineSpatialNodes(
+    nsDisplayListBuilder* aBuilder, const ActiveScrolledRoot* aASR,
+    nsDisplayItem* aItem) {
   if (!aASR) {
     // Recursion base case
     return Nothing();
   }
-  ScrollableLayerGuid::ViewID viewId = aASR->GetViewId();
-  Maybe<wr::WrSpatialId> space =
-      mBuilder->GetScrollIdForDefinedScrollLayer(viewId);
-  if (space) {
-    // If we've already defined this scroll layer before, we can early-exit
-    return space;
+
+  ScrollableLayerGuid::ViewID viewId = ScrollableLayerGuid::NULL_SCROLL_ID;
+  if (aASR->mKind == ActiveScrolledRoot::ASRKind::Scroll) {
+    viewId = aASR->GetViewId();
+    Maybe<wr::WrSpatialId> space =
+        mBuilder->GetScrollIdForDefinedScrollLayer(viewId);
+    if (space) {
+      // If we've already defined this scroll layer before, we can early-exit
+      return space;
+    }
   }
+
   // Recurse to define the ancestors
   Maybe<wr::WrSpatialId> ancestorSpace =
-      DefineScrollLayers(aASR->mParent, aItem);
+      DefineSpatialNodes(aBuilder, aASR->mParent, aItem);
 
-  ScrollContainerFrame* scrollContainerFrame = aASR->mScrollContainerFrame;
+  if (aASR->mKind == ActiveScrolledRoot::ASRKind::Sticky) {
+    Maybe<wr::WrSpatialId> parent = ancestorSpace.map(
+        [this](wr::WrSpatialId& aId) { return SpatialIdAfterOverride(aId); });
+    return ClipManager::DefineStickyNode(aBuilder, parent, aASR, aItem);
+  }
+
+  MOZ_ASSERT(viewId != ScrollableLayerGuid::NULL_SCROLL_ID);
+
+  ScrollContainerFrame* scrollContainerFrame = aASR->ScrollFrame();
   Maybe<ScrollMetadata> metadata = scrollContainerFrame->ComputeScrollMetadata(
       mManager, aItem->Frame(), aItem->ToReferenceFrame());
   if (!metadata) {
@@ -402,7 +679,7 @@ Maybe<wr::WrClipChainId> ClipManager::DefineClipChain(
   for (const DisplayItemClipChain* chain = aChain; chain;
        chain = chain->mParent) {
     MOZ_DIAGNOSTIC_ASSERT(chain->mOnStack || !chain->mASR ||
-                          chain->mASR->mScrollContainerFrame);
+                          chain->mASR->mFrame);
 
     if (!chain->mClip.HasClip()) {
       // This item in the chain is a no-op, skip over it
@@ -424,7 +701,7 @@ Maybe<wr::WrClipChainId> ClipManager::DefineClipChain(
     AutoTArray<wr::ComplexClipRegion, 6> wrRoundedRects;
     chain->mClip.ToComplexClipRegions(aAppUnitsPerDevPixel, wrRoundedRects);
 
-    wr::WrSpatialId space = GetScrollLayer(chain->mASR);
+    wr::WrSpatialId space = GetSpatialId(chain->mASR);
     // Define the clip
     space = SpatialIdAfterOverride(space);
 
