@@ -59,11 +59,12 @@ class LayerActivity {
   }
   ~LayerActivity();
   nsExpirationState* GetExpirationState() { return &mState; }
-  uint8_t& RestyleCountForProperty(nsCSSPropertyID aProperty) {
+  uint8_t& RestyleCountForProperty(NonCustomCSSPropertyId aProperty) {
     return mRestyleCounts[GetActivityIndexForProperty(aProperty)];
   }
 
-  static ActivityIndex GetActivityIndexForProperty(nsCSSPropertyID aProperty) {
+  static ActivityIndex GetActivityIndexForProperty(
+      NonCustomCSSPropertyId aProperty) {
     switch (aProperty) {
       case eCSSProperty_opacity:
         return ACTIVITY_OPACITY;
@@ -274,7 +275,7 @@ static void IncrementScaleRestyleCountIfNeeded(nsIFrame* aFrame,
 
 /* static */
 void ActiveLayerTracker::NotifyRestyle(nsIFrame* aFrame,
-                                       nsCSSPropertyID aProperty) {
+                                       NonCustomCSSPropertyId aProperty) {
   LayerActivity* layerActivity = GetLayerActivityForUpdate(aFrame);
   uint8_t& mutationCount = layerActivity->RestyleCountForProperty(aProperty);
   IncrementMutationCount(&mutationCount);
@@ -298,7 +299,7 @@ static bool IsPresContextInScriptAnimationCallback(
 
 /* static */
 void ActiveLayerTracker::NotifyInlineStyleRuleModified(
-    nsIFrame* aFrame, nsCSSPropertyID aProperty) {
+    nsIFrame* aFrame, NonCustomCSSPropertyId aProperty) {
   if (IsPresContextInScriptAnimationCallback(aFrame->PresContext())) {
     LayerActivity* layerActivity = GetLayerActivityForUpdate(aFrame);
     // We know this is animated, so just hack the mutation count.
