@@ -19,37 +19,47 @@ use crate::values::computed::font::GenericFontFamily;
 use crate::values::computed::{ColorScheme, Length, NonNegativeLength};
 use crate::values::specified::color::{ColorSchemeFlags, ForcedColors, SystemColor};
 use crate::values::specified::font::{
-    QueryFontMetricsFlags, FONT_MEDIUM_LINE_HEIGHT_PX, FONT_MEDIUM_PX,
+    QueryFontMetricsFlags, FONT_MEDIUM_CAP_PX, FONT_MEDIUM_CH_PX, FONT_MEDIUM_EX_PX,
+    FONT_MEDIUM_IC_PX, FONT_MEDIUM_LINE_HEIGHT_PX, FONT_MEDIUM_PX,
 };
 use crate::values::specified::ViewportVariant;
 use crate::values::{CustomIdent, KeyframesName};
 use app_units::{Au, AU_PER_PX};
 use euclid::default::Size2D;
 use euclid::{Scale, SideOffsets2D};
+use parking_lot::RwLock;
 use servo_arc::Arc;
 use std::sync::atomic::{AtomicBool, AtomicU32, AtomicUsize, Ordering};
-use std::{cmp, fmt};
+use std::{cmp, fmt, mem};
 use style_traits::{CSSPixel, DevicePixel};
 
 /// The `Device` in Gecko wraps a pres context, has a default values computed,
 /// and contains all the viewport rule state.
+///
+/// This structure also contains atomics used for computing root font-relative
+/// units. These atomics use relaxed ordering, since when computing the style
+/// of the root element, there can't be any other style being computed at the
+/// same time (given we need the style of the parent to compute everything else).
 pub struct Device {
     /// NB: The document owns the styleset, who owns the stylist, and thus the
     /// `Device`, so having a raw document pointer here is fine.
     document: *const structs::Document,
     default_values: Arc<ComputedValues>,
-    /// The font size of the root element.
-    ///
-    /// This is set when computing the style of the root element, and used for
-    /// rem units in other elements.
-    ///
-    /// When computing the style of the root element, there can't be any other
-    /// style being computed at the same time, given we need the style of the
-    /// parent to compute everything else. So it is correct to just use a
-    /// relaxed atomic here.
+    /// Current computed style of the root element, used for calculations of
+    /// root font-relative units.
+    root_style: RwLock<Arc<ComputedValues>>,
+    /// Font size of the root element, used for rem units in other elements.
     root_font_size: AtomicU32,
     /// Line height of the root element, used for rlh units in other elements.
     root_line_height: AtomicU32,
+    /// X-height of the root element, used for rex units in other elements.
+    root_font_metrics_ex: AtomicU32,
+    /// Cap-height of the root element, used for rcap units in other elements.
+    root_font_metrics_cap: AtomicU32,
+    /// Advance measure (ch) of the root element, used for rch units in other elements.
+    root_font_metrics_ch: AtomicU32,
+    /// Ideographic advance measure of the root element, used for ric units in other elements.
+    root_font_metrics_ic: AtomicU32,
     /// The body text color, stored as an `nscolor`, used for the "tables
     /// inherit from body" quirk.
     ///
@@ -61,6 +71,10 @@ pub struct Device {
     /// Whether any styles computed in the document relied on the root line-height
     /// by using rlh units.
     used_root_line_height: AtomicBool,
+    /// Whether any styles computed in the document relied on the root font metrics
+    /// by using rcap, rch, rex, or ric units. This is a lock instead of an atomic
+    /// in order to prevent concurrent writes to the root font metric values.
+    used_root_font_metrics: RwLock<bool>,
     /// Whether any styles computed in the document relied on font metrics.
     used_font_metrics: AtomicBool,
     /// Whether any styles computed in the document relied on the viewport size
@@ -98,16 +112,25 @@ impl Device {
         assert!(!document.is_null());
         let doc = unsafe { &*document };
         let prefs = unsafe { &*bindings::Gecko_GetPrefSheetPrefs(doc) };
+        let default_values = ComputedValues::default_values(doc);
+        let root_style = RwLock::new(Arc::clone(&default_values));
         Device {
             document,
-            default_values: ComputedValues::default_values(doc),
+            default_values: default_values,
+            root_style: root_style,
             root_font_size: AtomicU32::new(FONT_MEDIUM_PX.to_bits()),
             root_line_height: AtomicU32::new(FONT_MEDIUM_LINE_HEIGHT_PX.to_bits()),
+            root_font_metrics_ex: AtomicU32::new(FONT_MEDIUM_EX_PX.to_bits()),
+            root_font_metrics_cap: AtomicU32::new(FONT_MEDIUM_CAP_PX.to_bits()),
+            root_font_metrics_ch: AtomicU32::new(FONT_MEDIUM_CH_PX.to_bits()),
+            root_font_metrics_ic: AtomicU32::new(FONT_MEDIUM_IC_PX.to_bits()),
+
             // This gets updated when we see the <body>, so it doesn't really
             // matter which color-scheme we look at here.
             body_text_color: AtomicUsize::new(prefs.mLightColors.mDefault as usize),
             used_root_font_size: AtomicBool::new(false),
             used_root_line_height: AtomicBool::new(false),
+            used_root_font_metrics: RwLock::new(false),
             used_font_metrics: AtomicBool::new(false),
             used_viewport_size: AtomicBool::new(false),
             used_dynamic_viewport_size: AtomicBool::new(false),
@@ -168,6 +191,12 @@ impl Device {
         &self.default_values
     }
 
+    /// Store a pointer to the root element's computed style, for use in
+    /// calculation of root font-relative metrics.
+    pub fn set_root_style(&self, style: &Arc<ComputedValues>) {
+        *self.root_style.write() = style.clone();
+    }
+
     /// Get the font size of the root element (for rem)
     pub fn root_font_size(&self) -> Length {
         self.used_root_font_size.store(true, Ordering::Relaxed);
@@ -193,6 +222,66 @@ impl Device {
             .store(size.to_bits(), Ordering::Relaxed);
     }
 
+    /// Get the x-height of the root element (for rex)
+    pub fn root_font_metrics_ex(&self) -> Length {
+        self.ensure_root_font_metrics_updated();
+        Length::new(f32::from_bits(
+            self.root_font_metrics_ex.load(Ordering::Relaxed),
+        ))
+    }
+
+    /// Set the x-height of the root element (for rex), in zoom-independent CSS pixels.
+    pub fn set_root_font_metrics_ex(&self, size: f32) -> bool {
+        let size = size.to_bits();
+        let previous = self.root_font_metrics_ex.swap(size, Ordering::Relaxed);
+        previous != size
+    }
+
+    /// Get the cap-height of the root element (for rcap)
+    pub fn root_font_metrics_cap(&self) -> Length {
+        self.ensure_root_font_metrics_updated();
+        Length::new(f32::from_bits(
+            self.root_font_metrics_cap.load(Ordering::Relaxed),
+        ))
+    }
+
+    /// Set the cap-height of the root element (for rcap), in zoom-independent CSS pixels.
+    pub fn set_root_font_metrics_cap(&self, size: f32) -> bool {
+        let size = size.to_bits();
+        let previous = self.root_font_metrics_cap.swap(size, Ordering::Relaxed);
+        previous != size
+    }
+
+    /// Get the advance measure of the root element (for rch)
+    pub fn root_font_metrics_ch(&self) -> Length {
+        self.ensure_root_font_metrics_updated();
+        Length::new(f32::from_bits(
+            self.root_font_metrics_ch.load(Ordering::Relaxed),
+        ))
+    }
+
+    /// Set the advance measure of the root element (for rch), in zoom-independent CSS pixels.
+    pub fn set_root_font_metrics_ch(&self, size: f32) -> bool {
+        let size = size.to_bits();
+        let previous = self.root_font_metrics_ch.swap(size, Ordering::Relaxed);
+        previous != size
+    }
+
+    /// Get the ideographic advance measure of the root element (for ric)
+    pub fn root_font_metrics_ic(&self) -> Length {
+        self.ensure_root_font_metrics_updated();
+        Length::new(f32::from_bits(
+            self.root_font_metrics_ic.load(Ordering::Relaxed),
+        ))
+    }
+
+    /// Set the ideographic advance measure of the root element (for ric), in zoom-independent CSS pixels.
+    pub fn set_root_font_metrics_ic(&self, size: f32) -> bool {
+        let size = size.to_bits();
+        let previous = self.root_font_metrics_ic.swap(size, Ordering::Relaxed);
+        previous != size
+    }
+
     /// The quirks mode of the document.
     pub fn quirks_mode(&self) -> QuirksMode {
         self.document().mCompatMode.into()
@@ -230,8 +319,11 @@ impl Device {
         font: &crate::properties::style_structs::Font,
         base_size: Length,
         flags: QueryFontMetricsFlags,
+        track_usage: bool,
     ) -> FontMetrics {
-        self.used_font_metrics.store(true, Ordering::Relaxed);
+        if track_usage {
+            self.used_font_metrics.store(true, Ordering::Relaxed);
+        }
         let pc = match self.pres_context() {
             Some(pc) => pc,
             None => return Default::default(),
@@ -269,6 +361,55 @@ impl Device {
         }
     }
 
+    fn ensure_root_font_metrics_updated(&self) {
+        let mut guard = self.used_root_font_metrics.write();
+        let previously_computed = mem::replace(&mut *guard, true);
+        if !previously_computed {
+            self.update_root_font_metrics();
+        }
+    }
+
+    /// Compute the root element's font metrics, and returns a bool indicating whether
+    /// the font metrics have changed since the previous restyle.
+    pub fn update_root_font_metrics(&self) -> bool {
+        let root_style = self.root_style.read();
+        let root_effective_zoom = (*root_style).effective_zoom;
+        let root_font_size = (*root_style).get_font().clone_font_size().computed_size();
+
+        let root_font_metrics = self.query_font_metrics(
+            (*root_style).writing_mode.is_upright(),
+            &(*root_style).get_font(),
+            root_font_size,
+            QueryFontMetricsFlags::USE_USER_FONT_SET
+                | QueryFontMetricsFlags::NEEDS_CH
+                | QueryFontMetricsFlags::NEEDS_IC,
+            /* track_usage = */ false,
+        );
+
+        let mut root_font_metrics_changed = false;
+        root_font_metrics_changed |= self.set_root_font_metrics_ex(
+            root_effective_zoom.unzoom(root_font_metrics.x_height_or_default(root_font_size).px()),
+        );
+        root_font_metrics_changed |= self.set_root_font_metrics_ch(
+            root_effective_zoom.unzoom(
+                root_font_metrics
+                    .zero_advance_measure_or_default(
+                        root_font_size,
+                        (*root_style).writing_mode.is_upright(),
+                    )
+                    .px(),
+            ),
+        );
+        root_font_metrics_changed |= self.set_root_font_metrics_cap(
+            root_effective_zoom.unzoom(root_font_metrics.cap_height_or_default().px()),
+        );
+        root_font_metrics_changed |= self.set_root_font_metrics_ic(
+            root_effective_zoom.unzoom(root_font_metrics.ic_width_or_default(root_font_size).px()),
+        );
+
+        root_font_metrics_changed
+    }
+
     /// Returns the body text color.
     pub fn body_text_color(&self) -> AbsoluteColor {
         convert_nscolor_to_absolute_color(self.body_text_color.load(Ordering::Relaxed) as u32)
@@ -309,6 +450,7 @@ impl Device {
         self.reset_computed_values();
         self.used_root_font_size.store(false, Ordering::Relaxed);
         self.used_root_line_height.store(false, Ordering::Relaxed);
+        self.used_root_font_metrics = RwLock::new(false);
         self.used_font_metrics.store(false, Ordering::Relaxed);
         self.used_viewport_size.store(false, Ordering::Relaxed);
         self.used_dynamic_viewport_size
@@ -325,6 +467,11 @@ impl Device {
         self.used_root_line_height.load(Ordering::Relaxed)
     }
 
+    /// Returns whether we ever looked up the root font metrics of the device.
+    pub fn used_root_font_metrics(&self) -> bool {
+        *self.used_root_font_metrics.read()
+    }
+
     /// Recreates all the temporary state that the `Device` stores.
     ///
     /// This includes the viewport override from `@viewport` rules, and also the
