@@ -1348,6 +1348,18 @@ nsresult Database::InitSchema(bool* aDatabaseMigrated) {
 
       // Firefox 141 uses schema version 82
 
+      if (currentSchemaVersion < 83) {
+        rv = MigrateV83Up();
+        NS_ENSURE_SUCCESS(rv, rv);
+      }
+
+      if (currentSchemaVersion < 84) {
+        rv = MigrateV84Up();
+        NS_ENSURE_SUCCESS(rv, rv);
+      }
+
+      // Firefox 147 uses schema version 84
+
       // Schema Upgrades must add migration code here.
       // >>> IMPORTANT! <<<
       // NEVER MIX UP SYNC AND ASYNC EXECUTION IN MIGRATORS, YOU MAY LOCK THE
@@ -1721,8 +1733,6 @@ nsresult Database::InitFunctions(mozIStorageConnection* aMainConn) {
   NS_ENSURE_SUCCESS(rv, rv);
   rv = StripPrefixAndUserinfoFunction::create(aMainConn);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = IsFrecencyDecayingFunction::create(aMainConn);
-  NS_ENSURE_SUCCESS(rv, rv);
   rv = NoteSyncChangeFunction::create(aMainConn);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = InvalidateDaysOfHistoryFunction::create(aMainConn);
@@ -1808,26 +1818,38 @@ nsresult Database::InitTempEntities() {
   rv = mMainConn->ExecuteSimpleSQL(CREATE_PLACES_METADATA_AFTERDELETE_TRIGGER);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (StaticPrefs::places_frecency_pages_alternative_featureGate_AtStartup()) {
-    int32_t viewTimeMs =
-        StaticPrefs::
-            places_frecency_pages_alternative_interactions_viewTimeSeconds_AtStartup() *
-        1000;
-    int32_t viewTimeIfManyKeypressesMs =
-        StaticPrefs::
-            places_frecency_pages_alternative_interactions_viewTimeIfManyKeypressesSeconds_AtStartup() *
-        1000;
-    int32_t manyKeypresses = StaticPrefs::
-        places_frecency_pages_alternative_interactions_manyKeypresses_AtStartup();
+  // Thresholds chosen for elevating a visit to a higher bucket.
+  bool useAlternative =
+      StaticPrefs::places_frecency_pages_alternative_featureGate_AtStartup();
+  int32_t viewTimeMs =
+      (useAlternative
+           ? StaticPrefs::
+                     places_frecency_pages_alternative_interactions_viewTimeSeconds_AtStartup() *
+                 1000
+           : StaticPrefs::
+                     places_frecency_pages_interactions_viewTimeSeconds_AtStartup() *
+                 1000);
+  int32_t viewTimeIfManyKeypressesMs =
+      (useAlternative
+           ? StaticPrefs::
+                     places_frecency_pages_alternative_interactions_viewTimeIfManyKeypressesSeconds_AtStartup() *
+                 1000
+           : StaticPrefs::
+                     places_frecency_pages_interactions_viewTimeIfManyKeypressesSeconds_AtStartup() *
+                 1000);
+  int32_t manyKeypresses =
+      (useAlternative
+           ? StaticPrefs::
+                 places_frecency_pages_alternative_interactions_manyKeypresses_AtStartup()
+           : StaticPrefs::
+                 places_frecency_pages_interactions_manyKeypresses_AtStartup());
+  rv = mMainConn->ExecuteSimpleSQL(CREATE_PLACES_METADATA_AFTERINSERT_TRIGGER(
+      viewTimeMs, viewTimeIfManyKeypressesMs, manyKeypresses));
+  NS_ENSURE_SUCCESS(rv, rv);
 
-    rv = mMainConn->ExecuteSimpleSQL(CREATE_PLACES_METADATA_AFTERINSERT_TRIGGER(
-        viewTimeMs, viewTimeIfManyKeypressesMs, manyKeypresses));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    rv = mMainConn->ExecuteSimpleSQL(CREATE_PLACES_METADATA_AFTERUPDATE_TRIGGER(
-        viewTimeMs, viewTimeIfManyKeypressesMs, manyKeypresses));
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
+  rv = mMainConn->ExecuteSimpleSQL(CREATE_PLACES_METADATA_AFTERUPDATE_TRIGGER(
+      viewTimeMs, viewTimeIfManyKeypressesMs, manyKeypresses));
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // Create triggers to remove rows with empty json
   rv = mMainConn->ExecuteSimpleSQL(CREATE_MOZ_PLACES_EXTRA_AFTERUPDATE_TRIGGER);
@@ -2255,6 +2277,24 @@ nsresult Database::MigrateV82Up() {
   return NS_OK;
 }
 
+nsresult Database::MigrateV83Up() {
+  // Recalculate frecency due to changing calculate_frecency.
+  nsresult rv = mMainConn->ExecuteSimpleSQL(
+      "UPDATE moz_places SET recalc_frecency = 1 WHERE frecency > 0"_ns);
+  NS_ENSURE_SUCCESS(rv, rv);
+  return NS_OK;
+}
+
+nsresult Database::MigrateV84Up() {
+  // Recalculate frecency due to changing calculate_frecency.
+  nsresult rv = mMainConn->ExecuteSimpleSQL(
+      "UPDATE moz_origins "
+      "SET recalc_frecency = 1 "
+      "WHERE frecency > 1"_ns);
+  NS_ENSURE_SUCCESS(rv, rv);
+  return NS_OK;
+}
+
 int64_t Database::CreateMobileRoot() {
   MOZ_ASSERT(NS_IsMainThread());
 
