@@ -557,32 +557,6 @@ class StripPrefixAndUserinfoFunction final : public mozIStorageFunction {
   ~StripPrefixAndUserinfoFunction() = default;
 };
 
-////////////////////////////////////////////////////////////////////////////////
-//// Is frecency decaying function
-
-/**
- * Returns nsNavHistory::IsFrecencyDecaying().
- *
- * @return
- *        True if frecency is currently decaying and false otherwise.
- */
-class IsFrecencyDecayingFunction final : public mozIStorageFunction {
- public:
-  NS_DECL_THREADSAFE_ISUPPORTS
-  NS_DECL_MOZISTORAGEFUNCTION
-
-  /**
-   * Registers the function with the specified database connection.
-   *
-   * @param aDBConn
-   *        The database connection to register with.
-   */
-  static nsresult create(mozIStorageConnection* aDBConn);
-
- private:
-  ~IsFrecencyDecayingFunction() = default;
-};
-
 ////////////////////////////////////////////////////////////////////////////////
 //// Should start frecency recalculation function
 
